"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensureRemotePlatformIsConfigured = ensureRemotePlatformIsConfigured;
exports.setOutputChannel = setOutputChannel;
const vscode = require("vscode");
const os = require("os");
const path = require("path");
const fsp = require("fs/promises");
const fs = require("fs");
/**
 * Step0: 获取当前操作系统下 VS Code 的 settings.json 文件的绝对路径。
 */
function getSettingsPath() {
    const platform = os.platform();
    const homeDir = os.homedir();
    switch (platform) {
        case 'win32':
            // 通常在 %APPDATA%\Code\User\settings.json
            return path.join(process.env.APPDATA || path.join(homeDir, 'AppData', 'Roaming'), 'Code', 'User', 'settings.json');
        case 'darwin':
            // macOS: ~/Library/Application Support/Code/User/settings.json
            return path.join(homeDir, 'Library', 'Application Support', 'Code', 'User', 'settings.json');
        case 'linux':
            // Linux: ~/.config/Code/User/settings.json
            return path.join(homeDir, '.config', 'Code', 'User', 'settings.json');
        default:
            throw new Error(`不支持的平台: ${platform}`);
    }
}
/**
 * 确保 VS Code 的 settings.json 中为指定用户配置了正确的远程平台。
 * @param username - 当前登录的用户名。
 * @param postStatus - 用于向 UI 回传状态的函数。
 */
function ensureRemotePlatformIsConfigured(username, postStatus) {
    return __awaiter(this, void 0, void 0, function* () {
        const settingsPath = getSettingsPath();
        const hostName = `${username}.bigquant`;
        const remotePlatformKey = 'remote.SSH.remotePlatform';
        postStatus(`检查 VS Code 远程平台配置...`);
        outputChannel.appendLine(`Settings file path: ${settingsPath}`);
        try {
            let settingsJson = {};
            if (fs.existsSync(settingsPath)) {
                const rawContent = yield fsp.readFile(settingsPath, 'utf8');
                // 尝试解析 JSON，忽略注释以增加稳健性
                try {
                    const jsonWithoutComments = rawContent.replace(/\\"|"(?:\\"|[^"])*"|(\/\/.*|\/\*[\s\S]*?\*\/)/g, (m, g) => g ? "" : m);
                    settingsJson = JSON.parse(jsonWithoutComments);
                }
                catch (parseError) {
                    postStatus(`警告: settings.json 文件格式错误，将创建新配置。`);
                    outputChannel.appendLine(`Could not parse settings.json: ${parseError}`);
                }
            }
            let remotePlatformSettings = settingsJson[remotePlatformKey];
            if (typeof remotePlatformSettings !== 'object' || remotePlatformSettings === null) {
                remotePlatformSettings = {};
                settingsJson[remotePlatformKey] = remotePlatformSettings;
            }
            if (remotePlatformSettings[hostName] === 'linux') {
                postStatus("远程平台已正确配置。");
                outputChannel.appendLine(`Configuration for host '${hostName}' is already correct.`);
                return;
            }
            postStatus(`为 主机 '${hostName}' 添加 'linux' 平台配置。`);
            outputChannel.appendLine(`Adding 'linux' platform for host '${hostName}'.`);
            remotePlatformSettings[hostName] = 'linux';
            const newContent = JSON.stringify(settingsJson, null, 4);
            yield fsp.writeFile(settingsPath, newContent, 'utf8');
            postStatus("VS Code 配置已更新。");
            vscode.window.showInformationMessage("VS Code 远程配置已更新。如果连接仍然失败，请尝试重启 VS Code。", { modal: true });
        }
        catch (error) {
            postStatus(`错误: 无法自动更新 VS Code 配置。`);
            outputChannel.appendLine(`Failed to update settings.json: ${error.message}`);
            vscode.window.showErrorMessage(`无法自动更新 VS Code 配置文件 (settings.json)。请手动添加 '"remote.SSH.remotePlatform": { "${hostName}": "linux" }'。`);
            throw error; // 停止后续连接流程
        }
    });
}
// 在模块作用域内声明一个 outputChannel 变量，它将在 activate 函数中被赋值
let outputChannel;
function setOutputChannel(channel) {
    outputChannel = channel;
}
//# sourceMappingURL=configure-vscode-settings.js.map