"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensureDefaultTerminalProfileIsConfigured = ensureDefaultTerminalProfileIsConfigured;
const vscode = require("vscode");
const os = require("os");
const path = require("path");
const fsp = require("fs/promises");
const fs = require("fs");
/**
 * Gets the absolute path to the user's VS Code settings.json file.
 * @returns The absolute path to settings.json.
 */
function getSettingsPath() {
    const platform = os.platform();
    const homeDir = os.homedir();
    switch (platform) {
        case 'win32':
            return path.join(process.env.APPDATA || path.join(homeDir, 'AppData', 'Roaming'), 'Code', 'User', 'settings.json');
        case 'darwin':
            return path.join(homeDir, 'Library', 'Application Support', 'Code', 'User', 'settings.json');
        case 'linux':
            return path.join(homeDir, '.config', 'Code', 'User', 'settings.json');
        default:
            throw new Error(`Unsupported platform: ${platform}`);
    }
}
/**
 * Ensures that a default Linux terminal profile is configured in VS Code's settings.json.
 * This is crucial for ensuring `vscode.env.shell` is defined in remote contexts.
 * @param outputChannel The channel for logging detailed information.
 * @returns A promise that resolves to `true` if a change was made, and `false` otherwise.
 */
function ensureDefaultTerminalProfileIsConfigured(outputChannel, postStatus) {
    return __awaiter(this, void 0, void 0, function* () {
        const settingsPath = getSettingsPath();
        const settingKey = 'terminal.integrated.defaultProfile.linux';
        const defaultValue = 'bash';
        postStatus("检查 VS Code 默认终端配置...");
        outputChannel.appendLine(`Checking for '${settingKey}' in ${settingsPath}`);
        try {
            let settingsJson = {};
            if (fs.existsSync(settingsPath)) {
                const rawContent = yield fsp.readFile(settingsPath, 'utf8');
                // Use a robust method to parse JSON that may contain comments
                try {
                    const jsonWithoutComments = rawContent.replace(/\\"|"(?:\\"|[^"])*"|(\/{2}.*|\/\*[\s\S]*?\*\/)/g, (m, g) => g ? "" : m);
                    settingsJson = JSON.parse(jsonWithoutComments);
                }
                catch (parseError) {
                    postStatus("警告: settings.json 文件格式错误。");
                    outputChannel.appendLine(`Could not parse settings.json: ${parseError}`);
                    // Proceed with an empty object, preserving the broken file is not ideal.
                }
            }
            // If the setting is already present, do nothing.
            if (settingsJson[settingKey]) {
                postStatus("默认终端已配置。");
                outputChannel.appendLine(`'${settingKey}' is already set to '${settingsJson[settingKey]}'.`);
                return false; // No change was made
            }
            // Otherwise, add the default setting.
            postStatus(`设置默认 Linux 终端为 '${defaultValue}'...`);
            outputChannel.appendLine(`Setting '${settingKey}' to '${defaultValue}'.`);
            settingsJson[settingKey] = defaultValue;
            const newContent = JSON.stringify(settingsJson, null, 4);
            yield fsp.writeFile(settingsPath, newContent, 'utf8');
            postStatus("VS Code 配置已更新。");
            return true; // A change was made
        }
        catch (error) {
            postStatus("错误: 无法自动更新 VS Code 配置。");
            outputChannel.appendLine(`Failed to update settings.json: ${error.message}`);
            vscode.window.showErrorMessage(`无法自动更新 'settings.json'。请手动添加设置 '"terminal.integrated.defaultProfile.linux": "bash"'。`);
            throw error; // Stop the connection process
        }
    });
}
//# sourceMappingURL=configure-vscode-terminal.js.map