"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensureRemoteExtensionsAreInstalled = ensureRemoteExtensionsAreInstalled;
exports.clearExtensionCache = clearExtensionCache;
exports.cleanInvalidRemoteExtensions = cleanInvalidRemoteExtensions;
exports.validateAndCleanCache = validateAndCleanCache;
const vscode = require("vscode");
const fs = require("fs");
const path = require("path");
const os = require("os");
const configurations_1 = require("./configurations");
// 持久化缓存文件路径 - 使用更可靠的路径
const EXTENSION_CACHE_FILE = path.join(os.homedir(), '.ssh', 'remote-extensions-installed.json');
const INSTALLATION_COOLDOWN_FILE = path.join(os.homedir(), '.ssh', 'extension-install-cooldown.json');
// 安装冷却时间（秒）
const INSTALLATION_COOLDOWN_SECONDS = 30;
// 确保缓存目录存在
function ensureCacheDirectoryExists() {
    const cacheDir = path.dirname(EXTENSION_CACHE_FILE);
    if (!fs.existsSync(cacheDir)) {
        fs.mkdirSync(cacheDir, { recursive: true });
    }
}
// 获取或创建持久化缓存
function getInstalledExtensionsCache(outputChannel) {
    try {
        ensureCacheDirectoryExists();
        if (fs.existsSync(EXTENSION_CACHE_FILE)) {
            const cacheData = JSON.parse(fs.readFileSync(EXTENSION_CACHE_FILE, 'utf8'));
            if (outputChannel) {
                outputChannel.appendLine(`缓存文件路径: ${EXTENSION_CACHE_FILE}`);
                outputChannel.appendLine(`缓存文件内容: ${JSON.stringify(cacheData)}`);
            }
            return new Set(cacheData.installedExtensions || []);
        }
        else {
            if (outputChannel) {
                outputChannel.appendLine(`缓存文件不存在: ${EXTENSION_CACHE_FILE}`);
            }
        }
    }
    catch (error) {
        if (outputChannel) {
            outputChannel.appendLine(`读取缓存失败: ${error}`);
        }
        console.log('Failed to read extension cache:', error);
    }
    return new Set();
}
// 保存已安装插件到缓存
function saveInstalledExtensionsCache(installedExtensions, outputChannel) {
    try {
        ensureCacheDirectoryExists();
        const cacheData = {
            installedExtensions: Array.from(installedExtensions),
            lastUpdated: new Date().toISOString(),
            totalCount: installedExtensions.size
        };
        fs.writeFileSync(EXTENSION_CACHE_FILE, JSON.stringify(cacheData, null, 2), { mode: 0o600 });
        if (outputChannel) {
            outputChannel.appendLine(`✓ 缓存已保存到: ${EXTENSION_CACHE_FILE}`);
            outputChannel.appendLine(`✓ 缓存内容: ${JSON.stringify(cacheData)}`);
        }
    }
    catch (error) {
        if (outputChannel) {
            outputChannel.appendLine(`保存缓存失败: ${error}`);
        }
        console.log('Failed to save extension cache:', error);
    }
}
// 检查安装冷却时间
function isInstallationInCooldown(outputChannel) {
    try {
        if (!fs.existsSync(INSTALLATION_COOLDOWN_FILE)) {
            return false;
        }
        const cooldownData = JSON.parse(fs.readFileSync(INSTALLATION_COOLDOWN_FILE, 'utf8'));
        const lastInstallTime = new Date(cooldownData.lastInstallTime);
        const now = new Date();
        const timeDiffSeconds = (now.getTime() - lastInstallTime.getTime()) / 1000;
        if (timeDiffSeconds < INSTALLATION_COOLDOWN_SECONDS) {
            if (outputChannel) {
                const remainingMinutes = Math.ceil(INSTALLATION_COOLDOWN_SECONDS - timeDiffSeconds);
                outputChannel.appendLine(`安装插件过程冷却中，等待 ${remainingMinutes} 秒钟后重新扫描...`);
            }
            return true;
        }
        return false;
    }
    catch (error) {
        if (outputChannel) {
            outputChannel.appendLine(`检查冷却时间失败: ${error}`);
        }
        return false;
    }
}
// 设置安装冷却时间
function setInstallationCooldown(outputChannel) {
    try {
        ensureCacheDirectoryExists();
        const cooldownData = {
            lastInstallTime: new Date().toISOString()
        };
        fs.writeFileSync(INSTALLATION_COOLDOWN_FILE, JSON.stringify(cooldownData, null, 2), { mode: 0o600 });
        if (outputChannel) {
            outputChannel.appendLine(`✓ 设置安装冷却时间: ${INSTALLATION_COOLDOWN_SECONDS} 秒`);
        }
    }
    catch (error) {
        if (outputChannel) {
            outputChannel.appendLine(`设置冷却时间失败: ${error}`);
        }
    }
}
// 简化的远程插件检测函数
function detectRemoteExtension(extId, installedExtensions, outputChannel) {
    if (outputChannel) {
        outputChannel.appendLine(`=== 检测远程插件: ${extId} ===`);
    }
    // 方法1：检查是否在已安装列表中
    const directExtensionFound = installedExtensions.includes(extId);
    if (directExtensionFound) {
        if (outputChannel) {
            outputChannel.appendLine(`✓ 方法1成功: 在已安装列表中找到 ${extId}`);
        }
        return true;
    }
    // 方法2：使用vscode.extensions.getExtension检查
    const vsCodeExtension = vscode.extensions.getExtension(extId);
    if (vsCodeExtension) {
        if (outputChannel) {
            outputChannel.appendLine(`✓ 方法2成功: vscode.extensions.getExtension找到 ${extId}`);
            outputChannel.appendLine(`  - 插件激活状态: ${vsCodeExtension.isActive}`);
            outputChannel.appendLine(`  - 插件类型: ${vsCodeExtension.extensionKind}`);
        }
        return true;
    }
    // 方法3：对Python插件，进行更全面的检测
    if (extId === 'ms-python.python') {
        // 3.1 检查所有插件中是否有Python相关的
        const allExtensionsIds = vscode.extensions.all.map(ext => ext.id.toLowerCase());
        const pythonRelatedExtensions = allExtensionsIds.filter(ext => ext.includes('python') || ext === 'ms-python.python');
        if (outputChannel) {
            outputChannel.appendLine(`所有Python相关插件: [${pythonRelatedExtensions.join(', ')}]`);
        }
        if (pythonRelatedExtensions.length > 0) {
            if (outputChannel) {
                outputChannel.appendLine(`✓ 方法3.1成功: 在所有插件中找到Python相关插件`);
            }
            return true;
        }
        // 3.2 检查传入的installedExtensions列表
        const pythonInList = installedExtensions.filter(ext => ext.includes('python') || ext.includes('Python'));
        if (outputChannel) {
            outputChannel.appendLine(`传入列表中的Python插件: [${pythonInList.join(', ')}]`);
        }
        if (pythonInList.length > 0) {
            if (outputChannel) {
                outputChannel.appendLine(`✓ 方法3.2成功: 在传入列表中找到Python插件`);
            }
            return true;
        }
    }
    // 方法4：使用vscode.extensions.all进行深度搜索
    const allExtensions = vscode.extensions.all;
    const foundExtension = allExtensions.find(ext => ext.id.toLowerCase() === extId.toLowerCase());
    if (foundExtension) {
        if (outputChannel) {
            outputChannel.appendLine(`✓ 方法4成功: 在vscode.extensions.all中找到 ${extId}`);
            outputChannel.appendLine(`  - 是否内置: ${foundExtension.packageJSON.isBuiltin}`);
            outputChannel.appendLine(`  - 插件路径: ${foundExtension.extensionPath}`);
        }
        return true;
    }
    if (outputChannel) {
        outputChannel.appendLine(`✗ 所有方法都未找到插件: ${extId}`);
        outputChannel.appendLine(`=== 检测结束 ===`);
    }
    return false;
}
/**
 * 检查并安装远程环境中缺失的 VS Code 插件。
 * 使用 VS Code API 直接安装插件，避免依赖外部命令。
 */
function ensureRemoteExtensionsAreInstalled(outputChannel) {
    return __awaiter(this, void 0, void 0, function* () {
        outputChannel.appendLine("开始检查远程插件...");
        outputChannel.appendLine(`=== 插件检测会话开始: ${new Date().toISOString()} ===`);
        // 检查是否在冷却期内
        if (isInstallationInCooldown(outputChannel)) {
            vscode.window.showInformationMessage('插件安装功能成功进入冷却状态');
            outputChannel.appendLine(`=== 插件检测会话结束 (冷却中): ${new Date().toISOString()} ===`);
            return;
        }
        yield vscode.window.withProgress({
            location: vscode.ProgressLocation.Notification,
            title: "正在配置远程开发环境",
            cancellable: false
        }, (progress) => __awaiter(this, void 0, void 0, function* () {
            try {
                progress.report({ message: "正在获取已安装插件列表..." });
                // 首先清理缓存中不应在远程运行的插件
                cleanInvalidRemoteExtensions(outputChannel);
                // 获取持久化缓存
                const confirmedInstalledExtensions = getInstalledExtensionsCache(outputChannel);
                outputChannel.appendLine(`从缓存中找到 ${confirmedInstalledExtensions.size} 个已确认安装的插件`);
                if (confirmedInstalledExtensions.size > 0) {
                    outputChannel.appendLine(`缓存中的插件: ${Array.from(confirmedInstalledExtensions).join(', ')}`);
                }
                // 获取当前已安装的插件 - 改进检测逻辑
                const installedExtensionsDetailed = vscode.extensions.all
                    .filter(ext => !ext.packageJSON.isBuiltin)
                    .map(ext => ({
                    id: ext.id.toLowerCase(),
                    isActive: ext.isActive,
                    extensionKind: ext.extensionKind
                }));
                const installedExtensions = installedExtensionsDetailed.map(ext => ext.id);
                outputChannel.appendLine(`Total non-builtin extensions found: ${installedExtensions.length}`);
                outputChannel.appendLine(`Installed extensions: ${installedExtensions.join(', ')}`);
                // 改进的检测逻辑：多种方法综合检测
                const verifiedInstalledExtensions = [];
                for (const requiredExt of configurations_1.remoteExtensions) {
                    const extId = requiredExt.id.toLowerCase();
                    outputChannel.appendLine(`=== 检测插件: ${requiredExt.id} ===`);
                    let isInstalled = false;
                    // 方法1：检查是否在已安装列表中
                    if (installedExtensions.includes(extId)) {
                        outputChannel.appendLine(`✓ 方法1成功: 在已安装列表中找到 ${requiredExt.id}`);
                        isInstalled = true;
                    }
                    // 方法2：使用VS Code API检查
                    if (!isInstalled) {
                        let extension = vscode.extensions.getExtension(requiredExt.id);
                        // 对于特殊ID格式，尝试其他可能的变体
                        if (!extension && requiredExt.id.includes('python')) {
                            extension = vscode.extensions.getExtension('ms-python.python') ||
                                vscode.extensions.getExtension('MS-PYTHON.python');
                        }
                        if (extension) {
                            outputChannel.appendLine(`✓ 方法2成功: VS Code API找到 ${requiredExt.id} (Active: ${extension.isActive})`);
                            isInstalled = true;
                        }
                    }
                    // 方法3：使用detectRemoteExtension函数进行深度检测
                    if (!isInstalled) {
                        if (detectRemoteExtension(extId, installedExtensions, outputChannel)) {
                            outputChannel.appendLine(`✓ 方法3成功: 深度检测找到 ${requiredExt.id}`);
                            isInstalled = true;
                        }
                    }
                    if (isInstalled) {
                        outputChannel.appendLine(`✓ 插件已安装: ${requiredExt.id}`);
                        verifiedInstalledExtensions.push(extId);
                        confirmedInstalledExtensions.add(extId); // 更新缓存记录
                    }
                    else {
                        outputChannel.appendLine(`✗ 插件未安装: ${requiredExt.id}`);
                        confirmedInstalledExtensions.delete(extId); // 从缓存中移除
                    }
                    outputChannel.appendLine(`=== 检测完成: ${requiredExt.id} ===`);
                }
                outputChannel.appendLine(`Verified installed extensions: ${verifiedInstalledExtensions.join(', ')}`);
                // 更新缓存
                saveInstalledExtensionsCache(confirmedInstalledExtensions, outputChannel);
                // 找出需要安装的插件
                const extensionsToInstall = configurations_1.remoteExtensions.filter(ext => !verifiedInstalledExtensions.includes(ext.id.toLowerCase()));
                if (extensionsToInstall.length === 0) {
                    vscode.window.showInformationMessage('远程开发环境已是最新状态。');
                    outputChannel.appendLine("All required extensions are already installed and verified.");
                    outputChannel.appendLine(`=== 插件检测会话结束 (无需安装): ${new Date().toISOString()} ===`);
                    return;
                }
                outputChannel.appendLine(`Extensions to install/reinstall: ${extensionsToInstall.map(e => e.id).join(', ')}`);
                // 安装插件
                for (let i = 0; i < extensionsToInstall.length; i++) {
                    const extension = extensionsToInstall[i];
                    progress.report({
                        message: `正在安装插件: ${extension.name} (${i + 1}/${extensionsToInstall.length})`,
                        increment: (100 / extensionsToInstall.length)
                    });
                    try {
                        // 再次检查插件是否在此期间被安装（避免重复安装）
                        const preInstallCheck = vscode.extensions.getExtension(extension.id);
                        if (preInstallCheck) {
                            outputChannel.appendLine(`✓ Extension ${extension.id} detected during installation process, skipping`);
                            confirmedInstalledExtensions.add(extension.id.toLowerCase());
                            continue;
                        }
                        outputChannel.appendLine(`开始安装插件: ${extension.id}`);
                        yield vscode.commands.executeCommand('workbench.extensions.installExtension', extension.id);
                        outputChannel.appendLine(`Install command executed for ${extension.id}`);
                        // 等待安装完成
                        yield new Promise(resolve => setTimeout(resolve, 2000));
                        // 安装后立即验证是否成功 - 获取最新的插件列表
                        const updatedExtensions = vscode.extensions.all
                            .filter(ext => !ext.packageJSON.isBuiltin)
                            .map(ext => ext.id.toLowerCase());
                        const postInstallVerification = detectRemoteExtension(extension.id.toLowerCase(), updatedExtensions, outputChannel);
                        if (postInstallVerification) {
                            outputChannel.appendLine(`✓ 安装验证成功: ${extension.id} 已正确安装`);
                            confirmedInstalledExtensions.add(extension.id.toLowerCase());
                            // 立即保存到持久化缓存
                            saveInstalledExtensionsCache(confirmedInstalledExtensions, outputChannel);
                        }
                        else {
                            outputChannel.appendLine(`⚠ 安装验证失败: ${extension.id} 可能需要重新加载窗口`);
                            // 仍然添加到缓存，但标记为需要验证
                            confirmedInstalledExtensions.add(extension.id.toLowerCase());
                            saveInstalledExtensionsCache(confirmedInstalledExtensions, outputChannel);
                        }
                    }
                    catch (installError) {
                        outputChannel.appendLine(`Failed to install ${extension.id}: ${installError}`);
                        vscode.window.showWarningMessage(`插件 ${extension.name} 安装失败，请手动安装`);
                    }
                }
                // 检查是否有插件被安装
                const wasAnyExtensionInstalled = extensionsToInstall.length > 0;
                // 保存最终的缓存状态
                saveInstalledExtensionsCache(confirmedInstalledExtensions, outputChannel);
                if (wasAnyExtensionInstalled) {
                    // 设置安装冷却时间
                    setInstallationCooldown(outputChannel);
                    // 如果安装了插件，提示用户选择是否重新加载
                    outputChannel.appendLine(`成功安装了 ${extensionsToInstall.length} 个插件`);
                    const reloadChoice = yield vscode.window.showWarningMessage(`已安装 ${extensionsToInstall.length} 个插件。建议重新加载窗口以确保插件正常工作。`, { modal: true }, '立即重新加载', '稍后手动重新加载');
                    if (reloadChoice === '立即重新加载') {
                        outputChannel.appendLine("用户选择立即重新加载窗口");
                        yield vscode.commands.executeCommand('workbench.action.reloadWindow');
                    }
                    else {
                        outputChannel.appendLine("用户选择稍后重新加载");
                        vscode.window.showInformationMessage('提醒：新插件需要重新加载窗口后才能正常工作');
                    }
                }
                else {
                    vscode.window.showInformationMessage('远程开发环境已是最新状态！');
                }
                outputChannel.appendLine(`=== 插件检测会话结束 (完成): ${new Date().toISOString()} ===`);
                // 只有当没有重新加载窗口时才进行延迟验证
                if (wasAnyExtensionInstalled) {
                    setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                        outputChannel.appendLine("--- 延迟验证插件状态 ---");
                        let allVerified = true;
                        const currentCache = getInstalledExtensionsCache(outputChannel);
                        let cacheUpdated = false;
                        for (const ext of configurations_1.remoteExtensions) {
                            const extension = vscode.extensions.getExtension(ext.id);
                            if (extension) {
                                outputChannel.appendLine(`✓ 延迟验证成功: ${ext.id} (Active: ${extension.isActive})`);
                                if (!currentCache.has(ext.id.toLowerCase())) {
                                    currentCache.add(ext.id.toLowerCase());
                                    cacheUpdated = true;
                                }
                            }
                            else {
                                outputChannel.appendLine(`⚠ 延迟验证失败: ${ext.id} - 插件可能仍在加载`);
                                allVerified = false;
                            }
                        }
                        if (cacheUpdated) {
                            saveInstalledExtensionsCache(currentCache, outputChannel);
                            outputChannel.appendLine("已更新持久化缓存");
                        }
                        if (!allVerified) {
                            outputChannel.appendLine("建议：如果插件仍未正常工作，请手动重新加载窗口（Ctrl+Shift+P -> Developer: Reload Window）");
                        }
                        else {
                            outputChannel.appendLine("✓ 所有插件验证成功！");
                        }
                        outputChannel.appendLine("--- 延迟验证完成 ---");
                    }), 5000); // 5秒后验证（减少等待时间）
                }
            }
            catch (error) {
                const errorMessage = error instanceof Error ? error.message : String(error);
                outputChannel.appendLine(`FATAL ERROR: ${errorMessage}`);
                vscode.window.showErrorMessage(`配置远程环境时发生错误: ${errorMessage}`, { modal: true });
            }
        }));
    });
}
/**
 * 清除插件安装缓存，用于强制重新检查所有插件
 */
function clearExtensionCache() {
    try {
        if (fs.existsSync(EXTENSION_CACHE_FILE)) {
            fs.unlinkSync(EXTENSION_CACHE_FILE);
            console.log('Extension cache cleared successfully');
        }
        // 同时清除冷却时间文件
        if (fs.existsSync(INSTALLATION_COOLDOWN_FILE)) {
            fs.unlinkSync(INSTALLATION_COOLDOWN_FILE);
            console.log('Installation cooldown cleared successfully');
        }
    }
    catch (error) {
        console.log('Failed to clear extension cache:', error);
    }
}
/**
 * 清理缓存中不应在远程安装的插件（如语言包）
 */
function cleanInvalidRemoteExtensions(outputChannel) {
    try {
        const cachedExtensions = getInstalledExtensionsCache(outputChannel);
        const validExtensions = new Set();
        let cleaned = false;
        for (const extId of cachedExtensions) {
            // 移除语言包等不应在远程运行的插件
            if (extId.includes('language-pack')) {
                outputChannel.appendLine(`移除不应在远程运行的插件: ${extId}`);
                cleaned = true;
            }
            else {
                validExtensions.add(extId);
            }
        }
        if (cleaned) {
            saveInstalledExtensionsCache(validExtensions, outputChannel);
            outputChannel.appendLine(`已清理无效的远程插件缓存记录`);
        }
    }
    catch (error) {
        outputChannel.appendLine(`清理缓存失败: ${error}`);
    }
}
/**
 * 验证并清理缓存，移除不再存在的插件
 */
function validateAndCleanCache(outputChannel) {
    try {
        const cachedExtensions = getInstalledExtensionsCache(outputChannel);
        if (cachedExtensions.size === 0) {
            outputChannel.appendLine("缓存为空，无需验证");
            return;
        }
        outputChannel.appendLine("开始验证缓存准确性...");
        const cleanedCache = new Set();
        let removedCount = 0;
        for (const extId of cachedExtensions) {
            const extension = vscode.extensions.getExtension(extId);
            const currentExtensions = vscode.extensions.all
                .filter(ext => !ext.packageJSON.isBuiltin)
                .map(ext => ext.id.toLowerCase());
            if (extension || currentExtensions.includes(extId)) {
                cleanedCache.add(extId);
            }
            else {
                outputChannel.appendLine(`移除缓存中不存在的插件: ${extId}`);
                removedCount++;
            }
        }
        if (removedCount > 0) {
            saveInstalledExtensionsCache(cleanedCache, outputChannel);
            outputChannel.appendLine(`缓存验证完成，移除了 ${removedCount} 个不存在的插件记录`);
        }
        else {
            outputChannel.appendLine("缓存验证完成，所有记录都准确");
        }
    }
    catch (error) {
        outputChannel.appendLine(`缓存验证失败: ${error}`);
    }
}
//# sourceMappingURL=install-remote-extensions.js.map