"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.activate = activate;
exports.deactivate = deactivate;
// src/extension.ts
const vscode = require("vscode");
const os = require("os");
const fsp = require("fs/promises");
const fs = require("fs");
const path = require("path");
const axios_1 = require("axios");
// 导入所有模块化函数
const check_dependencies_1 = require("./core/check-dependencies");
const initial_cleanup_1 = require("./core/initial-cleanup");
const setup_ssh_key_1 = require("./core/setup-ssh-key");
const download_socat_1 = require("./core/download-socat");
const extract_and_rename_1 = require("./core/extract-and-rename");
const configure_ssh_1 = require("./core/configure-ssh");
const upload_public_key_1 = require("./core/upload-public-key");
const open_aistudio_1 = require("./core/open-aistudio");
const establish_connection_1 = require("./core/establish-connection");
const install_remote_extensions_1 = require("./core/install-remote-extensions");
const IS_LOGGED_IN_KEY = "bigquantconnector.isLoggedIn";
// 插件激活入口
function activate(context) {
    const outputChannel = vscode.window.createOutputChannel("BigQuant Connector");
    outputChannel.appendLine("BigQuant Connector is starting...");
    // =================================================================================
    // 远程环境逻辑：执行插件安装检查
    // =================================================================================
    if (vscode.env.remoteName === 'ssh-remote') {
        outputChannel.appendLine("插件在远程环境被成功激活。");
        // 延迟执行插件检查，确保远程环境完全加载
        setTimeout(() => __awaiter(this, void 0, void 0, function* () {
            try {
                outputChannel.appendLine("正在检查远程插件...");
                yield (0, install_remote_extensions_1.ensureRemoteExtensionsAreInstalled)(outputChannel);
                outputChannel.appendLine("远程插件检查完成。");
            }
            catch (err) {
                const errorMessage = err instanceof Error ? err.message : String(err);
                outputChannel.appendLine(`远程插件安装过程中发生错误: ${errorMessage}`);
                vscode.window.showErrorMessage("检查或安装远程插件时发生错误，请检查输出日志。");
            }
        }), 3000); // 延迟3秒执行
        return; // 在远程环境中，只执行此逻辑
    }
    // =================================================================================
    // 本地环境逻辑：加载UI和连接功能
    // =================================================================================
    outputChannel.appendLine("插件在本地环境被激活。");
    const isLoggedIn = !!context.globalState.get('username');
    vscode.commands.executeCommand('setContext', IS_LOGGED_IN_KEY, isLoggedIn);
    const provider = new BigQuantViewProvider(context);
    context.subscriptions.push(vscode.window.registerWebviewViewProvider('bigquant.primaryView', provider));
    context.subscriptions.push(vscode.commands.registerCommand('bigquantconnector.logout', () => provider.logout()));
    // 注册手动安装远程插件的命令
    context.subscriptions.push(vscode.commands.registerCommand('bigquantconnector.installRemoteExtensions', () => __awaiter(this, void 0, void 0, function* () {
        if (vscode.env.remoteName === 'ssh-remote') {
            try {
                yield (0, install_remote_extensions_1.ensureRemoteExtensionsAreInstalled)(outputChannel);
            }
            catch (err) {
                const errorMessage = err instanceof Error ? err.message : String(err);
                outputChannel.appendLine(`手动安装远程插件失败: ${errorMessage}`);
                vscode.window.showErrorMessage("手动安装远程插件失败，请检查输出日志。");
            }
        }
        else {
            vscode.window.showInformationMessage("此命令只能在远程环境中使用。");
        }
    })));
    // 注册清除插件缓存的命令
    context.subscriptions.push(vscode.commands.registerCommand('bigquantconnector.clearExtensionCache', () => {
        (0, install_remote_extensions_1.clearExtensionCache)();
        outputChannel.appendLine("插件安装缓存已清除");
        vscode.window.showInformationMessage("插件安装缓存已清除，下次检查时将重新扫描所有插件。");
    }));
    // 注册验证和清理缓存的命令
    context.subscriptions.push(vscode.commands.registerCommand('bigquantconnector.validateExtensionCache', () => {
        (0, install_remote_extensions_1.validateAndCleanCache)(outputChannel);
        vscode.window.showInformationMessage("插件缓存验证完成，请查看输出日志了解详情。");
    }));
}
class BigQuantViewProvider {
    constructor(_context) {
        this._context = _context;
        this._outputChannel = vscode.window.createOutputChannel("BigQuant Connector");
    }
    resolveWebviewView(webviewView) {
        this._view = webviewView;
        webviewView.webview.options = {
            enableScripts: true,
            localResourceRoots: [vscode.Uri.joinPath(this._context.extensionUri, 'src')]
        };
        webviewView.webview.html = this._getHtmlForWebview();
        webviewView.webview.onDidReceiveMessage((message) => __awaiter(this, void 0, void 0, function* () {
            switch (message.command) {
                case 'ui-ready':
                    this.updateLoginView();
                    break;
                case 'login':
                    yield this.handleLogin(message.payload.username, message.payload.password);
                    break;
                case 'logout':
                    yield this.logout();
                    break;
                case 'connect':
                    yield this.handleConnect();
                    break;
            }
        }));
    }
    updateLoginView() {
        const username = this._context.globalState.get('username');
        if (username) {
            this.postToWebview({ command: 'showConnectionView', payload: { username } });
        }
        else {
            this.postToWebview({ command: 'showLoginView' });
        }
    }
    postToWebview(message) {
        var _a;
        (_a = this._view) === null || _a === void 0 ? void 0 : _a.webview.postMessage(message);
    }
    handleLogin(username, password) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!username || !password) {
                this.postToWebview({ command: 'login-error', text: '用户名和密码不能为空' });
                return;
            }
            try {
                const res = yield axios_1.default.post('https://bigquant.com/bigapis/auth/v1/auth/login', {
                    payload: { username, password },
                    auth_name: "UserPass"
                });
                if (res.data.code === 0) {
                    yield this._context.globalState.update("username", res.data.data.user.username);
                    yield this._context.secrets.store("token", res.data.data.token);
                    yield this._context.secrets.store("id", res.data.data.user.id);
                    yield vscode.commands.executeCommand('setContext', IS_LOGGED_IN_KEY, true);
                    this.updateLoginView();
                }
                else {
                    this.postToWebview({ command: 'login-error', text: res.data.message || '登录失败' });
                }
            }
            catch (error) {
                this.postToWebview({ command: 'login-error', text: '登录失败，请检查网络、账户或密码' });
            }
        });
    }
    logout() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield axios_1.default.delete('https://bigquant.com/bigapis/auth/v1/auth/logout');
            }
            catch (error) {
                console.error("Logout API call failed, but proceeding with local logout.", error);
            }
            yield this._context.globalState.update("username", undefined);
            yield this._context.secrets.delete("token");
            yield this._context.secrets.delete("id");
            yield vscode.commands.executeCommand('setContext', IS_LOGGED_IN_KEY, false);
            vscode.window.showInformationMessage('已退出登录');
            this.postToWebview({ command: 'showLoginView' });
        });
    }
    handleConnect() {
        return __awaiter(this, void 0, void 0, function* () {
            const postStatus = (text) => this.postToWebview({ command: 'update-status', text });
            try {
                const username = this._context.globalState.get("username");
                if (!username) {
                    postStatus("错误：无法获取用户信息，请重新登录。");
                    return;
                }
                const dependenciesReady = yield (0, check_dependencies_1.checkAndInstallDependencies)(postStatus);
                if (!dependenciesReady)
                    return;
                const hasUsedFile = path.join(os.homedir(), '.ssh', 'hasUsed.dat');
                let registeredUsernames = fs.existsSync(hasUsedFile) ? yield fsp.readFile(hasUsedFile, 'utf8') : "";
                if (!registeredUsernames.includes(username)) {
                    postStatus("首次连接，开始初始化环境...");
                    yield (0, initial_cleanup_1.performInitialCleanup)(postStatus);
                    yield (0, setup_ssh_key_1.setupSshKey)(postStatus);
                    yield (0, download_socat_1.downloadSocat)(postStatus);
                    yield (0, extract_and_rename_1.extractAndRenameSocat)(postStatus);
                    yield (0, configure_ssh_1.configureSshProxy)(username, postStatus);
                    yield fsp.appendFile(hasUsedFile, `\n${username}`, { mode: 0o600 });
                    postStatus("初始化完成！准备连接...");
                }
                yield (0, upload_public_key_1.uploadPublicKey)(this._context, postStatus);
                yield (0, open_aistudio_1.openAiStudio)(this._context, postStatus);
                yield (0, establish_connection_1.establishConnection)(username, postStatus);
            }
            catch (error) {
                const errorMessage = error.message || String(error);
                postStatus(`错误: ${errorMessage}`);
                vscode.window.showErrorMessage(`连接过程中发生错误: ${errorMessage}`);
            }
            finally {
                this.postToWebview({ command: 'connect-finished' });
            }
        });
    }
    _getHtmlForWebview() {
        const htmlPath = path.join(this._context.extensionPath, 'src', 'ui.html');
        return fs.readFileSync(htmlPath, 'utf-8');
    }
}
BigQuantViewProvider.viewType = 'bigquant.primaryView';
function deactivate() {
    vscode.commands.executeCommand('setContext', IS_LOGGED_IN_KEY, false);
}
//# sourceMappingURL=extension.js.map