import * as vscode from 'vscode';
import * as os from 'os';
import * as fs from 'fs';
import axios from 'axios';
import { exec } from 'child_process';
import { socatDownloadUrl, socatZipPath, hasDownloadedFlagPath, hasUsedFlagPath, socatTargetDirName } from './configurations';

/**
 * Step4: 根据不同操作系统处理 socat 连接工具。
 */
export async function downloadSocat(postStatus: (text: string) => void): Promise<void> {
    const platform = os.platform();
    postStatus(`检查操作系统，当前为: ${platform}`);

    switch (platform) {
        case 'win32':
            if (!fs.existsSync(socatTargetDirName) || !fs.existsSync(hasDownloadedFlagPath) || !fs.existsSync(hasUsedFlagPath)) {
                postStatus("在 Windows 上下载连接工具(socat)...");
                
                await vscode.window.withProgress({ location: vscode.ProgressLocation.Notification, title: `Downloading socat...`, cancellable: false }, async (progress) => {
                    const response = await axios({ method: 'GET', url: socatDownloadUrl, responseType: 'stream' });
                    if (response.status >= 200 && response.status < 300) {
                        const totalBytes = response.headers['content-length'] ? parseInt(response.headers['content-length'], 10) : 0;
                        let downloadedBytes = 0;
                        const writer = fs.createWriteStream(socatZipPath);
                        (response.data as NodeJS.ReadableStream).on('data', (chunk: Buffer) => {
                            downloadedBytes += chunk.length;
                            if (totalBytes > 0) { progress.report({ increment: (chunk.length / totalBytes) * 100 }); }
                        });
                        (response.data as NodeJS.ReadableStream).pipe(writer);
                        await new Promise<void>((resolve, reject) => {
                            writer.on('finish', resolve);
                            writer.on('error', reject);
                        });
                        vscode.window.showInformationMessage(`下载成功: socat.zip`);
                    } else {
                        throw new Error(`下载失败，服务器状态: ${response.status}`);
                    }
                });
            }
            break;

        case 'darwin': // macOS
            postStatus("在 macOS 上检查 Homebrew...");

            const installSocatWithBrew = () => {
                return new Promise<void>((resolve, reject) => {
                    postStatus("使用 Homebrew 安装 socat...");
                    vscode.window.withProgress({ location: vscode.ProgressLocation.Notification, title: 'Installing socat via Homebrew...', cancellable: false }, async () => {
                        const command = 'HOMEBREW_NO_AUTO_UPDATE=1 brew install socat';
                        exec(command, (installError, installStdout, installStderr) => {
                            if (installError) {
                                const errorMsg = `socat 安装失败: ${installStderr}`;
                                vscode.window.showErrorMessage(errorMsg);
                                postStatus(errorMsg);
                                reject(new Error(errorMsg));
                            } else {
                                vscode.window.showInformationMessage('socat 安装成功!');
                                postStatus("socat 安装成功。");
                                resolve();
                            }
                        });
                    });
                });
            };

            await new Promise<void>((resolve, reject) => {
                exec('command -v brew', (error, stdout) => {
                    if (error || !stdout) {
                        postStatus("Homebrew 未安装，将尝试自动安装...");
                        vscode.window.showInformationMessage("Homebrew 未安装。将在新终端中开始安装，请根据提示操作（可能需要输入密码���。");

                        const terminal = vscode.window.createTerminal("Homebrew Installation");
                        terminal.show();
                        terminal.sendText('/bin/bash -c "$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/HEAD/install.sh)"');
                        terminal.sendText('echo "Homebrew 安装进程已启动。完成后请手动关闭此终端以继续。"');

                        const disposable = vscode.window.onDidCloseTerminal(closedTerminal => {
                            if (closedTerminal === terminal) {
                                disposable.dispose();
                                postStatus("Homebrew 安装终端已关闭，正在验证安装结果...");
                                exec('command -v brew', (verifyError, verifyStdout) => {
                                    if (verifyError || !verifyStdout) {
                                        const errorMsg = "Homebrew 安装失败或被取消。请手动安装后再试。";
                                        vscode.window.showErrorMessage(errorMsg);
                                        postStatus(errorMsg);
                                        reject(new Error(errorMsg));
                                    } else {
                                        postStatus("Homebrew 安装成功！");
                                        installSocatWithBrew().then(resolve).catch(reject);
                                    }
                                });
                            }
                        });
                    } else {
                        postStatus("Homebrew 已安装。");
                        installSocatWithBrew().then(resolve).catch(reject);
                    }
                });
            });
            break;

        case 'linux':
            postStatus("在 Linux 上尝试安装 socat...");
            await vscode.window.withProgress({ location: vscode.ProgressLocation.Notification, title: 'Installing socat on Linux...', cancellable: false }, async () => {
                await new Promise<void>((resolve, reject) => {
                    // 尝试使用 apt-get (Debian/Ubuntu)
                    exec('sudo apt-get update && sudo apt-get install -y socat', (error, stdout, stderr) => {
                        if (!error) {
                            vscode.window.showInformationMessage('socat 安装成功 (apt-get)!');
                            postStatus("socat 安装成功。");
                            resolve();
                            return;
                        }

                        // 如果 apt-get 失败，尝试使用 yum (CentOS/RHEL)
                        postStatus("apt-get failed, trying yum...");
                        exec('sudo yum install -y socat', (error2, stdout2, stderr2) => {
                            if (!error2) {
                                vscode.window.showInformationMessage('socat 安装成功 (yum)!');
                                postStatus("socat 安装成功。");
                                resolve();
                                return;
                            }

                            // 如果两者都失败
                            const errorMessage = '无法自动安装 socat。请使用您发行版的包管理器手动安装(例如: "sudo apt-get install socat" 或 "sudo yum install socat")。';
                            vscode.window.showErrorMessage(errorMessage);
                            postStatus(errorMessage);
                            reject(new Error(errorMessage));
                        });
                    });
                });
            });
            break;

        default:
            const unsupportedMessage = `不支持的操作系统: ${platform}`;
            postStatus(unsupportedMessage);
            vscode.window.showErrorMessage(unsupportedMessage);
            throw new Error(unsupportedMessage);
    }
}
