
// src/extension.ts
import * as vscode from 'vscode';
import * as os from 'os';
import * as fsp from 'fs/promises';
import * as fs from 'fs';
import * as path from 'path';
import axios from 'axios';

// 导入所有模块化函数
import { checkAndInstallDependencies } from './core/check-dependencies';
import { performInitialCleanup } from './core/initial-cleanup';
import { setupSshKey } from './core/setup-ssh-key';
import { downloadSocat } from './core/download-socat';
import { extractAndRenameSocat } from './core/extract-and-rename';
import { configureSshProxy } from './core/configure-ssh';
import { uploadPublicKey } from './core/upload-public-key';
import { openAiStudio } from './core/open-aistudio';
import { establishConnection } from './core/establish-connection';
import { ensureRemoteExtensionsAreInstalled, clearExtensionCache, validateAndCleanCache } from './core/install-remote-extensions';

const IS_LOGGED_IN_KEY = "bigquantconnector.isLoggedIn";

// --- 接口定义 ---
interface ApiResponse<T> {
    code: number;
    message: string;
    data: T;
}
interface User {
    token: string;
    user: { id: string; username: string; };
}

// 插件激活入口
export function activate(context: vscode.ExtensionContext) {
    const outputChannel = vscode.window.createOutputChannel("BigQuant Connector");
    outputChannel.appendLine("BigQuant Connector is starting...");

    // =================================================================================
    // 远程环境逻辑：执行插件安装检查
    // =================================================================================
    if (vscode.env.remoteName === 'ssh-remote') {
        outputChannel.appendLine("插件在远程环境被成功激活。");

        // 延迟执行插件检查，确保远程环境完全加载
        setTimeout(async () => {
            try {
                outputChannel.appendLine("正在检查远程插件...");
                await ensureRemoteExtensionsAreInstalled(outputChannel);
                outputChannel.appendLine("远程插件检查完成。");
            } catch (err) {
                const errorMessage = err instanceof Error ? err.message : String(err);
                outputChannel.appendLine(`远程插件安装过程中发生错误: ${errorMessage}`);
                vscode.window.showErrorMessage("检查或安装远程插件时发生错误，请检查输出日志。");
            }
        }, 3000); // 延迟3秒执行
        
        return; // 在远程环境中，只执行此逻辑
    }

    // =================================================================================
    // 本地环境逻辑：加载UI和连接功能
    // =================================================================================
    outputChannel.appendLine("插件在本地环境被激活。");
    const isLoggedIn = !!context.globalState.get<string>('username');
    vscode.commands.executeCommand('setContext', IS_LOGGED_IN_KEY, isLoggedIn);

    const provider = new BigQuantViewProvider(context);
    context.subscriptions.push(
        vscode.window.registerWebviewViewProvider('bigquant.primaryView', provider)
    );
    context.subscriptions.push(
        vscode.commands.registerCommand('bigquantconnector.logout', () => provider.logout())
    );
    
    // 注册手动安装远程插件的命令
    context.subscriptions.push(
        vscode.commands.registerCommand('bigquantconnector.installRemoteExtensions', async () => {
            if (vscode.env.remoteName === 'ssh-remote') {
                try {
                    await ensureRemoteExtensionsAreInstalled(outputChannel);
                } catch (err) {
                    const errorMessage = err instanceof Error ? err.message : String(err);
                    outputChannel.appendLine(`手动安装远程插件失败: ${errorMessage}`);
                    vscode.window.showErrorMessage("手动安装远程插件失败，请检查输出日志。");
                }
            } else {
                vscode.window.showInformationMessage("此命令只能在远程环境中使用。");
            }
        })
    );
    
    // 注册清除插件缓存的命令
    context.subscriptions.push(
        vscode.commands.registerCommand('bigquantconnector.clearExtensionCache', () => {
            clearExtensionCache();
            outputChannel.appendLine("插件安装缓存已清除");
            vscode.window.showInformationMessage("插件安装缓存已清除，下次检查时将重新扫描所有插件。");
        })
    );
    
    // 注册验证和清理缓存的命令
    context.subscriptions.push(
        vscode.commands.registerCommand('bigquantconnector.validateExtensionCache', () => {
            validateAndCleanCache(outputChannel);
            vscode.window.showInformationMessage("插件缓存验证完成，请查看输出日志了解详情。");
        })
    );
}

class BigQuantViewProvider implements vscode.WebviewViewProvider {
    public static readonly viewType = 'bigquant.primaryView';
    private _view?: vscode.WebviewView;
    private _outputChannel: vscode.OutputChannel;

    constructor(private readonly _context: vscode.ExtensionContext) {
        this._outputChannel = vscode.window.createOutputChannel("BigQuant Connector");
    }

    public resolveWebviewView(webviewView: vscode.WebviewView) {
        this._view = webviewView;
        webviewView.webview.options = {
            enableScripts: true,
            localResourceRoots: [vscode.Uri.joinPath(this._context.extensionUri, 'src')]
        };
        webviewView.webview.html = this._getHtmlForWebview();

        webviewView.webview.onDidReceiveMessage(async (message) => {
            switch (message.command) {
                case 'ui-ready':
                    this.updateLoginView();
                    break;
                case 'login':
                    await this.handleLogin(message.payload.username, message.payload.password);
                    break;
                case 'logout':
                    await this.logout();
                    break;
                case 'connect':
                    await this.handleConnect();
                    break;
            }
        });
    }

    private updateLoginView() {
        const username = this._context.globalState.get<string>('username');
        if (username) {
            this.postToWebview({ command: 'showConnectionView', payload: { username } });
        } else {
            this.postToWebview({ command: 'showLoginView' });
        }
    }

    private postToWebview(message: any) {
        this._view?.webview.postMessage(message);
    }

    public async handleLogin(username?: string, password?: string) {
        if (!username || !password) {
            this.postToWebview({ command: 'login-error', text: '用户名和密码不能为空' });
            return;
        }
        try {
            const res = await axios.post<ApiResponse<User>>('https://bigquant.com/bigapis/auth/v1/auth/login', {
                payload: { username, password },
                auth_name: "UserPass"
            });
            if (res.data.code === 0) {
                await this._context.globalState.update("username", res.data.data.user.username);
                await this._context.secrets.store("token", res.data.data.token);
                await this._context.secrets.store("id", res.data.data.user.id);
                await vscode.commands.executeCommand('setContext', IS_LOGGED_IN_KEY, true);
                this.updateLoginView();
            } else {
                this.postToWebview({ command: 'login-error', text: res.data.message || '登录失败' });
            }
        } catch (error) {
            this.postToWebview({ command: 'login-error', text: '登录失败，请检查网络、账户或密码' });
        }
    }

    public async logout() {
        try {
            await axios.delete('https://bigquant.com/bigapis/auth/v1/auth/logout');
        } catch (error) {
            console.error("Logout API call failed, but proceeding with local logout.", error);
        }
        await this._context.globalState.update("username", undefined);
        await this._context.secrets.delete("token");
        await this._context.secrets.delete("id");
        await vscode.commands.executeCommand('setContext', IS_LOGGED_IN_KEY, false);
        vscode.window.showInformationMessage('已退出登录');
        this.postToWebview({ command: 'showLoginView' });
    }

    public async handleConnect() {
        const postStatus = (text: string) => this.postToWebview({ command: 'update-status', text });

        try {

            const username = this._context.globalState.get<string>("username");
            if (!username) {
                postStatus("错误：无法获取用户信息，请重新登录。");
                return;
            }
            
            const dependenciesReady = await checkAndInstallDependencies(postStatus);
            if (!dependenciesReady) return;
            
            const hasUsedFile = path.join(os.homedir(), '.ssh', 'hasUsed.dat');
            let registeredUsernames = fs.existsSync(hasUsedFile) ? await fsp.readFile(hasUsedFile, 'utf8') : "";
            
            if (!registeredUsernames.includes(username)) {
                postStatus("首次连接，开始初始化环境...");
                await performInitialCleanup(postStatus);
                await setupSshKey(postStatus);
                await downloadSocat(postStatus);
                await extractAndRenameSocat(postStatus);
                await configureSshProxy(username, postStatus);
                
                await fsp.appendFile(hasUsedFile, `\n${username}`, { mode: 0o600 });
                postStatus("初始化完成！准备连接...");
            }
            
            await uploadPublicKey(this._context, postStatus);
            await openAiStudio(this._context, postStatus);
            await establishConnection(username, postStatus);

        } catch (error: any) {
            const errorMessage = error.message || String(error);
            postStatus(`错误: ${errorMessage}`);
            vscode.window.showErrorMessage(`连接过程中发生错误: ${errorMessage}`);
        } finally {
            this.postToWebview({ command: 'connect-finished' });
        }
    }
    
    private _getHtmlForWebview(): string {
        const htmlPath = path.join(this._context.extensionPath, 'src', 'ui.html');
        return fs.readFileSync(htmlPath, 'utf-8');
    }
}

export function deactivate() {
    vscode.commands.executeCommand('setContext', IS_LOGGED_IN_KEY, false);
}
